@@ -76,8 +76,8 @@ struct TrackerMinerFilesPrivate {
 
 #if defined(HAVE_UPOWER) || defined(HAVE_HAL)
 	TrackerPower *power;
-	gulong finished_handler;
 #endif /* defined(HAVE_UPOWER) || defined(HAVE_HAL) */
+	gulong finished_handler;
 
 	DBusGProxy *extractor_proxy;
 
@@ -85,6 +85,10 @@ struct TrackerMinerFilesPrivate {
 	GQuark quark_directory_config_root;
 
 	guint force_recheck_id;
+
+	gboolean index_removable_devices;
+	gboolean index_optical_discs;
+	guint volumes_changed_id;
 };
 
 enum {
@@ -97,8 +101,6 @@ enum {
 	PROP_CONFIG
 };
 
-static gchar *crawl_timestamp_file = NULL;
-
 static void        miner_files_set_property             (GObject              *object,
                                                          guint                 param_id,
                                                          const GValue         *value,
@@ -142,9 +144,13 @@ static void        index_recursive_directories_cb       (GObject              *g
 static void        index_single_directories_cb          (GObject              *gobject,
                                                          GParamSpec           *arg1,
                                                          gpointer              user_data);
+static gboolean    miner_files_force_recheck_idle       (gpointer user_data);
 static void        ignore_directories_cb                (GObject              *gobject,
 							 GParamSpec           *arg1,
 							 gpointer              user_data);
+static void        index_volumes_changed_cb             (GObject              *gobject,
+							 GParamSpec           *arg1,
+							 gpointer              user_data);
 static DBusGProxy *extractor_create_proxy               (void);
 static gboolean    miner_files_check_file               (TrackerMinerFS       *fs,
                                                          GFile                *file);
@@ -165,8 +171,8 @@ static gboolean    miner_files_ignore_next_update_file  (TrackerMinerFS       *f
                                                          GCancellable         *cancellable);
 static void        miner_files_finished                 (TrackerMinerFS       *fs);
 
-static void      extractor_get_embedded_metadata_cancel (GCancellable    *cancellable,
-                                                         ProcessFileData *data);
+static void        extractor_get_embedded_metadata_cancel (GCancellable    *cancellable,
+                                                           ProcessFileData *data);
 
 static void        miner_finished_cb                    (TrackerMinerFS *fs,
                                                          gdouble         seconds_elapsed,
@@ -176,6 +182,13 @@ static void        miner_finished_cb                    (TrackerMinerFS *fs,
                                                          guint           total_files_ignored,
                                                          gpointer        user_data);
 
+static gboolean    miner_files_remove_by_type           (TrackerMinerFiles  *miner,
+                                                         TrackerStorageType  type);
+
+static void        miner_files_add_removable_or_optical_directory (TrackerMinerFiles *mf,
+                                                                   const gchar       *mount_path,
+                                                                   const gchar       *uuid);
+
 G_DEFINE_TYPE (TrackerMinerFiles, tracker_miner_files, TRACKER_TYPE_MINER_FS)
 
 static void
@@ -236,13 +249,12 @@ tracker_miner_files_init (TrackerMinerFiles *mf)
 	g_signal_connect (priv->power, "notify::on-battery",
 	                  G_CALLBACK (battery_status_cb),
 	                  mf);
+#endif /* defined(HAVE_UPOWER) || defined(HAVE_HAL) */
 
 	priv->finished_handler = g_signal_connect_after (mf, "finished",
 							 G_CALLBACK (miner_finished_cb),
 							 NULL);
 
-#endif /* defined(HAVE_UPOWER) || defined(HAVE_HAL) */
-
 	priv->volume_monitor = g_volume_monitor_get ();
 	g_signal_connect (priv->volume_monitor, "mount-pre-unmount",
 	                  G_CALLBACK (mount_pre_unmount_cb),
@@ -352,9 +364,6 @@ miner_files_constructed (GObject *object)
 	TrackerMinerFS *fs;
 	GSList *dirs;
 	GSList *mounts = NULL, *m;
-	gboolean index_removable_devices;
-	gboolean index_optical_discs;
-	TrackerStorageType type = 0;
 
 	G_OBJECT_CLASS (tracker_miner_files_parent_class)->constructed (object);
 
@@ -366,18 +375,28 @@ miner_files_constructed (GObject *object)
 		return;
 	}
 
-	if (tracker_config_get_index_removable_devices (mf->private->config)) {
-		index_removable_devices = TRUE;
-		type |= TRACKER_STORAGE_REMOVABLE;
+	/* Setup initial flag for removable devices */
+	mf->private->index_removable_devices = tracker_config_get_index_removable_devices (mf->private->config);
+	if (mf->private->index_removable_devices) {
+		/* Get list of roots for removable devices (excluding optical) */
+		mounts = tracker_storage_get_device_roots (mf->private->storage,
+		                                           TRACKER_STORAGE_REMOVABLE,
+		                                           TRUE);
 	}
 
-	if (tracker_config_get_index_optical_discs (mf->private->config)) {
-		index_optical_discs = TRUE;
-		type |= TRACKER_STORAGE_OPTICAL;
+	/* Setup initial flag for optical discs. Note that if removable devices not indexed,
+	 * optical discs will also never be indexed */
+	mf->private->index_optical_discs = (mf->private->index_removable_devices ?
+	                                    tracker_config_get_index_optical_discs (mf->private->config) :
+	                                    FALSE);
+	if (mf->private->index_optical_discs) {
+		/* Get list of roots for removable+optical devices */
+		m = tracker_storage_get_device_roots (mf->private->storage,
+		                                      TRACKER_STORAGE_OPTICAL | TRACKER_STORAGE_REMOVABLE,
+		                                      TRUE);
+		mounts = g_slist_concat (mounts, m);
 	}
 
-	mounts = tracker_storage_get_device_roots (mf->private->storage, type, TRUE);
-
 #if defined(HAVE_UPOWER) || defined(HAVE_HAL)
 	check_battery_status (mf);
 #endif /* defined(HAVE_UPOWER) || defined(HAVE_HAL) */
@@ -482,31 +501,24 @@ miner_files_constructed (GObject *object)
 	/* Add mounts */
 	g_message ("Setting up directories to iterate from devices/discs");
 
-	if (!index_removable_devices) {
+	if (!mf->private->index_removable_devices) {
 		g_message ("  Removable devices are disabled in the config");
+
+		/* Make sure we don't have any resource in a volume of the given type */
+		miner_files_remove_by_type (mf, TRACKER_STORAGE_REMOVABLE);
 	}
 
-	if (!index_optical_discs) {
+	if (!mf->private->index_optical_discs) {
 		g_message ("  Optical discs are disabled in the config");
+
+		/* Make sure we don't have any resource in a volume of the given type */
+		miner_files_remove_by_type (mf, TRACKER_STORAGE_REMOVABLE | TRACKER_STORAGE_OPTICAL);
 	}
 
 	for (m = mounts; m; m = m->next) {
-		GFile *file = g_file_new_for_path (m->data);
-		const gchar *uuid = tracker_storage_get_uuid_for_file (mf->private->storage, file);
-
-		g_object_set_qdata_full (G_OBJECT (file),
-					 mf->private->quark_mount_point_uuid,
-					 g_strdup (uuid),
-					 (GDestroyNotify) g_free);
-		g_object_set_qdata (G_OBJECT (file),
-		                    mf->private->quark_directory_config_root,
-		                    GINT_TO_POINTER (TRUE));
-
-		g_message ("  Adding:'%s'", (gchar*) m->data);
-		tracker_miner_fs_directory_add (TRACKER_MINER_FS (mf),
-		                                file,
-		                                TRUE);
-		g_object_unref (file);
+		miner_files_add_removable_or_optical_directory (mf,
+		                                                (gchar *) m->data,
+		                                                NULL);
 	}
 
 	/* We want to get notified when config changes */
@@ -526,6 +538,12 @@ miner_files_constructed (GObject *object)
 	g_signal_connect (mf->private->config, "notify::ignored-directories-with-content",
 	                  G_CALLBACK (ignore_directories_cb),
 	                  mf);
+	g_signal_connect (mf->private->config, "notify::index-removable-devices",
+	                  G_CALLBACK (index_volumes_changed_cb),
+	                  mf);
+	g_signal_connect (mf->private->config, "notify::index-optical-discs",
+	                  G_CALLBACK (index_volumes_changed_cb),
+	                  mf);
 
 #if defined(HAVE_UPOWER) || defined(HAVE_HAL)
 
@@ -550,21 +568,53 @@ set_up_mount_point_cb (GObject      *source,
                        gpointer      user_data)
 {
 	gchar *removable_device_urn = user_data;
-
 	GError *error = NULL;
+
 	tracker_miner_execute_update_finish (TRACKER_MINER (source),
 	                                     result,
 	                                     &error);
 
 	if (error) {
-		g_critical ("Could not set up mount point '%s': %s",
-		            removable_device_urn, error->message);
+		g_critical ("Could not set mount point in database '%s', %s",
+		            removable_device_urn,
+			    error->message);
 		g_error_free (error);
 	}
 
 	g_free (removable_device_urn);
 }
 
+static void
+set_up_mount_point_type (TrackerMinerFiles *miner,
+                         const gchar       *removable_device_urn,
+                         gboolean           removable,
+                         gboolean           optical,
+                         GString           *accumulator)
+{
+	if (!accumulator) {
+		return;
+	}
+
+	g_debug ("Mount point type being set in DB for URN '%s'",
+	         removable_device_urn);
+
+	g_string_append_printf (accumulator,
+	                        "DELETE FROM <%s> { <%s> tracker:isRemovable ?unknown } WHERE { <%s> a tracker:Volume; tracker:isRemovable ?unknown } ",
+	                        removable_device_urn, removable_device_urn, removable_device_urn);
+
+	g_string_append_printf (accumulator,
+	                        "INSERT INTO <%s> { <%s> a tracker:Volume; tracker:isRemovable %s } ",
+	                        removable_device_urn, removable_device_urn, removable ? "true" : "false");
+
+	g_string_append_printf (accumulator,
+	                        "DELETE FROM <%s> { <%s> tracker:isOptical ?unknown } WHERE { <%s> a tracker:Volume; tracker:isOptical ?unknown } ",
+	                        removable_device_urn, removable_device_urn, removable_device_urn);
+
+	g_string_append_printf (accumulator,
+	                        "INSERT INTO <%s> { <%s> a tracker:Volume; tracker:isOptical %s } ",
+	                        removable_device_urn, removable_device_urn, optical ? "true" : "false");
+}
+
 static void
 set_up_mount_point (TrackerMinerFiles *miner,
                     const gchar       *removable_device_urn,
@@ -574,8 +624,7 @@ set_up_mount_point (TrackerMinerFiles *miner,
 {
 	GString *queries;
 
-	g_debug ("Setting mount point '%s' state in database (URN '%s')",
-	         mount_point,
+	g_debug ("Mount point state being set in DB for URN '%s'",
 	         removable_device_urn);
 
 	queries = g_string_new (NULL);
@@ -598,7 +647,7 @@ set_up_mount_point (TrackerMinerFiles *miner,
 			                        removable_device_urn, removable_device_urn, uri);
 
 			g_string_append_printf (queries,
-			                        "DROP GRAPH <%s> "
+			                        "DELETE { <%s> a rdfs:Resource }  "
 			                        "INSERT INTO <%s> { "
 			                        "  <%s> a tracker:Volume; "
 			                        "       tracker:mountPoint ?u "
@@ -761,28 +810,56 @@ query_mount_points_cb (GObject      *source,
 		if ((state & VOLUME_MOUNTED) &&
 		    !(state & VOLUME_MOUNTED_IN_STORE)) {
 			const gchar *mount_point = NULL;
+			TrackerStorageType type = 0;
 
+			/* Note: is there any case where the urn doesn't have our
+			 *  datasource prefix? */
 			if (g_str_has_prefix (urn, TRACKER_DATASOURCE_URN_PREFIX)) {
 				const gchar *uuid;
 
 				uuid = urn + strlen (TRACKER_DATASOURCE_URN_PREFIX);
 				mount_point = tracker_storage_get_mount_point_for_uuid (priv->storage, uuid);
+				type = tracker_storage_get_type_for_uuid (priv->storage, uuid);
 			}
 
 			if (urn) {
 				if (mount_point) {
-					g_debug ("URN '%s' (mount point: %s) was not reported to be mounted, but now it is, updating state",
-					         urn, mount_point);
+					g_debug ("Mount point state incorrect in DB for URN '%s', "
+						 "currently it is mounted on '%s'",
+					         urn,
+						 mount_point);
 				} else {
-					g_debug ("URN '%s' was not reported to be mounted, but now it is, updating state", urn);
+					g_debug ("Mount point state incorrect in DB for URN '%s', "
+						 "currently it is mounted",
+					         urn);
 				}
-				set_up_mount_point (TRACKER_MINER_FILES (miner), urn, mount_point, TRUE, accumulator);
+
+				/* Set mount point state */
+				set_up_mount_point (TRACKER_MINER_FILES (miner),
+						    urn,
+						    mount_point,
+						    TRUE,
+						    accumulator);
+
+				/* Set mount point type */
+				set_up_mount_point_type (TRACKER_MINER_FILES (miner),
+				                         urn,
+				                         TRACKER_STORAGE_TYPE_IS_REMOVABLE (type),
+				                         TRACKER_STORAGE_TYPE_IS_OPTICAL (type),
+				                         accumulator);
+
 			}
 		} else if (!(state & VOLUME_MOUNTED) &&
 		           (state & VOLUME_MOUNTED_IN_STORE)) {
 			if (urn) {
-				g_debug ("URN '%s' was reported to be mounted, but it isn't anymore, updating state", urn);
-				set_up_mount_point (TRACKER_MINER_FILES (miner), urn, NULL, FALSE, accumulator);
+				g_debug ("Mount point state incorrect in DB for URN '%s', "
+					 "currently it is NOT mounted",
+					 urn);
+				set_up_mount_point (TRACKER_MINER_FILES (miner),
+						    urn,
+						    NULL,
+						    FALSE,
+						    accumulator);
 			}
 		}
 	}
@@ -820,13 +897,22 @@ mount_point_removed_cb (TrackerStorage *storage,
 {
 	TrackerMinerFiles *miner = user_data;
 	gchar *urn;
-
-	g_debug ("Removing mount point '%s'", mount_point);
+	GFile *mount_point_file;
 
 	urn = g_strdup_printf (TRACKER_DATASOURCE_URN_PREFIX "%s", uuid);
+	g_debug ("Mount point removed for URN '%s'", urn);
 
+	mount_point_file = g_file_new_for_path (mount_point);
+
+	/* Set mount point status in tracker-store */
 	set_up_mount_point (miner, urn, mount_point, FALSE, NULL);
+
+	/* Tell TrackerMinerFS to skip monitoring everything under the mount
+	 *  point (in case there was no pre-unmount notification) */
+	tracker_miner_fs_directory_remove (TRACKER_MINER_FS (miner), mount_point_file);
+
 	g_free (urn);
+	g_object_unref (mount_point_file);
 }
 
 static void
@@ -839,52 +925,92 @@ mount_point_added_cb (TrackerStorage *storage,
 {
 	TrackerMinerFiles *miner = user_data;
 	TrackerMinerFilesPrivate *priv;
-	GFile *file;
 	gchar *urn;
-	gboolean index_removable_devices;
-	gboolean index_optical_discs;
-	gboolean should_crawl;
+	GString *queries;
 
 	priv = TRACKER_MINER_FILES_GET_PRIVATE (miner);
 
-	index_removable_devices = tracker_config_get_index_removable_devices (priv->config);
-	index_optical_discs = tracker_config_get_index_optical_discs (priv->config);
-
-	g_message ("Added mount point '%s'", mount_point);
-
-	should_crawl = TRUE;
-
-	if (removable && !tracker_config_get_index_removable_devices (priv->config)) {
-		g_message ("  Not crawling, removable devices disabled in config");
-		should_crawl = FALSE;
-	}
-
-	if (optical && !tracker_config_get_index_optical_discs (priv->config)) {
-		g_message ("  Not crawling, optical devices discs disabled in config");
-		should_crawl = FALSE;
-	}
-
-	if (should_crawl) {
-		g_message ("  Adding directory to crawler's queue");
-
-		file = g_file_new_for_path (mount_point);
-		g_object_set_qdata_full (G_OBJECT (file),
-		                         priv->quark_mount_point_uuid,
-		                         g_strdup (uuid),
-		                         (GDestroyNotify) g_free);
-
-		g_object_set_qdata (G_OBJECT (file),
-		                    priv->quark_directory_config_root,
-		                    GINT_TO_POINTER (TRUE));
-
-		tracker_miner_fs_directory_add (TRACKER_MINER_FS (user_data),
-		                                file,
-		                                TRUE);
-		g_object_unref (file);
-	}
-
 	urn = g_strdup_printf (TRACKER_DATASOURCE_URN_PREFIX "%s", uuid);
-	set_up_mount_point (miner, urn, mount_point, TRUE, NULL);
+	g_message ("Mount point added for URN '%s'", urn);
+
+	if (removable && !priv->index_removable_devices) {
+		g_message ("  Not crawling, removable devices disabled in config");
+	} else if (optical && !priv->index_optical_discs) {
+		g_message ("  Not crawling, optical devices discs disabled in config");
+	} else if (!removable && !optical) {
+		GFile *mount_point_file;
+		GSList *l;
+
+		mount_point_file = g_file_new_for_path (mount_point);
+
+		/* Check if one of the recursively indexed locations is in
+		 *   the mounted path, or if the mounted path is inside
+		 *   a recursively indexed directory... */
+		for (l = tracker_config_get_index_recursive_directories (miner->private->config);
+		     l;
+		     l = g_slist_next (l)) {
+			GFile *config_file;
+
+			config_file = g_file_new_for_path (l->data);
+
+			if (g_file_equal (config_file, mount_point_file) ||
+			    g_file_has_prefix (config_file, mount_point_file)) {
+				/* If the config path is contained inside the mount path,
+				 *  then add the config path to re-check */
+				g_message ("  Re-check of configured path '%s' needed (recursively)",
+				           (gchar *) l->data);
+				tracker_miner_fs_directory_add (TRACKER_MINER_FS (user_data),
+				                                config_file,
+				                                TRUE);
+			} else if (g_file_has_prefix (mount_point_file, config_file)) {
+				/* If the mount path is contained inside the config path,
+				 *  then add the mount path to re-check */
+				g_message ("  Re-check of path '%s' needed (inside configured path '%s')",
+				           mount_point,
+				           (gchar *) l->data);
+				tracker_miner_fs_directory_add (TRACKER_MINER_FS (user_data),
+				                                mount_point_file,
+				                                TRUE);
+			}
+			g_object_unref (config_file);
+		}
+
+		/* Check if one of the non-recursively indexed locations is in
+		 *  the mount path... */
+		for (l = tracker_config_get_index_single_directories (miner->private->config);
+		     l;
+		     l = g_slist_next (l)) {
+			GFile *config_file;
+
+			config_file = g_file_new_for_path (l->data);
+			if (g_file_equal (config_file, mount_point_file) ||
+			    g_file_has_prefix (config_file, mount_point_file)) {
+				g_message ("  Re-check of configured path '%s' needed (non-recursively)",
+				           (gchar *) l->data);
+				tracker_miner_fs_directory_add (TRACKER_MINER_FS (user_data),
+				                                config_file,
+				                                FALSE);
+			}
+			g_object_unref (config_file);
+		}
+
+		g_object_unref (mount_point_file);
+	} else {
+		g_message ("  Adding directories in removable/optical media to crawler's queue");
+		miner_files_add_removable_or_optical_directory (miner,
+		                                                mount_point,
+		                                                uuid);
+	}
+
+	queries = g_string_new ("");
+	set_up_mount_point (miner, urn, mount_point, TRUE, queries);
+	set_up_mount_point_type (miner, urn, removable, optical, queries);
+	tracker_miner_execute_update (TRACKER_MINER (miner),
+	                              queries->str,
+	                              NULL,
+	                              set_up_mount_point_cb,
+	                              g_strdup (urn));
+	g_string_free (queries, TRUE);
 	g_free (urn);
 }
 
@@ -936,7 +1062,7 @@ check_battery_status (TrackerMinerFiles *mf)
 			if (!tracker_config_get_index_on_battery_first_time (mf->private->config)) {
 				g_message ("Running on battery, but not enabled, pausing");
 				should_pause = TRUE;
-			} else if (tracker_db_manager_get_first_index_done()) {
+			} else if (tracker_db_manager_get_first_index_done ()) {
 				g_message ("Running on battery and first-time index "
 				           "already done, pausing");
 				should_pause = TRUE;
@@ -1016,7 +1142,9 @@ miner_finished_cb (TrackerMinerFS *fs,
 		mf->private->finished_handler = 0;
 	}
 
+#if defined(HAVE_UPOWER) || defined(HAVE_HAL)
 	check_battery_status (mf);
+#endif /* defined(HAVE_UPOWER) || defined(HAVE_HAL) */
 }
 
 #endif /* defined(HAVE_UPOWER) || defined(HAVE_HAL) */
@@ -1027,10 +1155,14 @@ mount_pre_unmount_cb (GVolumeMonitor    *volume_monitor,
                       TrackerMinerFiles *mf)
 {
 	GFile *mount_root;
+	gchar *uri;
 
 	mount_root = g_mount_get_root (mount);
+	uri = g_file_get_uri (mount_root);
+	g_message ("Pre-unmount requested for '%s'", uri);
 	tracker_miner_fs_directory_remove (TRACKER_MINER_FS (mf), mount_root);
 	g_object_unref (mount_root);
+	g_free (uri);
 }
 
 static gboolean
@@ -1164,10 +1296,11 @@ update_directories_from_new_config (TrackerMinerFS *mf,
                 if (!tracker_string_in_gslist (path, new_dirs)) {
                         GFile *file;
 
-                        g_message ("  Removing directory:'%s'", path);
+                        g_message ("  Removing directory: '%s'", path);
 
                         file = g_file_new_for_path (path);
-                        tracker_miner_fs_directory_remove (TRACKER_MINER_FS (mf), file);
+                        /* Fully remove item (monitors and from store) */
+                        tracker_miner_fs_directory_remove_full (TRACKER_MINER_FS (mf), file);
                         g_object_unref (file);
                 }
         }
@@ -1266,13 +1399,143 @@ static void
 ignore_directories_cb (GObject    *gobject,
                        GParamSpec *arg1,
                        gpointer    user_data)
+{
+	TrackerMinerFiles *mf = user_data;
+
+	if (mf->private->force_recheck_id == 0) {
+		/* Set idle so multiple changes in the config lead to one recheck */
+		mf->private->force_recheck_id =
+			g_idle_add (miner_files_force_recheck_idle, mf);
+	}
+}
+
+static gboolean
+index_volumes_changed_idle (gpointer user_data)
+{
+	TrackerMinerFiles *mf = user_data;
+	GSList *mounts_removed = NULL;
+	GSList *mounts_added = NULL;
+	gboolean new_index_removable_devices;
+	gboolean new_index_optical_discs;
+
+	g_message ("Volume-related configuration changed, updating...");
+
+	/* Read new config values. Note that if removable devices is FALSE,
+	 * optical discs will also always be FALSE. */
+	new_index_removable_devices = tracker_config_get_index_removable_devices (mf->private->config);
+	new_index_optical_discs = (new_index_removable_devices ?
+	                           tracker_config_get_index_optical_discs (mf->private->config) :
+	                           FALSE);
+
+	/* Removable devices config changed? */
+	if (mf->private->index_removable_devices != new_index_removable_devices) {
+		GSList *m;
+
+		/* Get list of roots for currently mounted removable devices
+		 * (excluding optical) */
+		m = tracker_storage_get_device_roots (mf->private->storage,
+		                                      TRACKER_STORAGE_REMOVABLE,
+		                                      TRUE);
+		/* Set new config value */
+		mf->private->index_removable_devices = new_index_removable_devices;
+
+		if (mf->private->index_removable_devices) {
+			/* If previously not indexing and now indexing, need to re-check
+			 * current mounted volumes, add new monitors and index new files
+			 */
+			mounts_added = m;
+		} else {
+			/* If previously indexing and now not indexing, need to re-check
+			 * current mounted volumes, remove monitors and remove all resources
+			 * from the store belonging to a removable device
+			 */
+			mounts_removed = m;
+
+			/* And now, single sparql update to remove all resources
+			 * corresponding to removable devices (includes those
+			 * not currently mounted) */
+			miner_files_remove_by_type (mf, TRACKER_STORAGE_REMOVABLE);
+		}
+	}
+
+	/* Optical discs config changed? */
+	if (mf->private->index_optical_discs != new_index_optical_discs) {
+		GSList *m;
+
+		/* Get list of roots for removable devices (excluding optical) */
+		m = tracker_storage_get_device_roots (mf->private->storage,
+		                                      TRACKER_STORAGE_REMOVABLE | TRACKER_STORAGE_OPTICAL,
+		                                      TRUE);
+
+		/* Set new config value */
+		mf->private->index_optical_discs = new_index_optical_discs;
+
+		if (mf->private->index_optical_discs) {
+			/* If previously not indexing and now indexing, need to re-check
+			 * current mounted volumes, add new monitors and index new files
+			 */
+			mounts_added = g_slist_concat (mounts_added, m);
+		} else {
+			/* If previously indexing and now not indexing, need to re-check
+			 * current mounted volumes, remove monitors and remove all resources
+			 * from the store belonging to a optical disc
+			 */
+			mounts_removed = g_slist_concat (mounts_removed, m);
+
+			/* And now, single sparql update to remove all resources
+			 * corresponding to removable+optical devices (includes those
+			 * not currently mounted) */
+			miner_files_remove_by_type (mf, TRACKER_STORAGE_REMOVABLE | TRACKER_STORAGE_OPTICAL);
+		}
+	}
+
+	/* Tell TrackerMinerFS to stop monitoring the given removed mount paths, if any */
+	if (mounts_removed) {
+		GSList *sl;
+
+		for (sl = mounts_removed; sl; sl = g_slist_next (sl)) {
+			GFile *mount_point_file;
+
+			mount_point_file = g_file_new_for_path (sl->data);
+			tracker_miner_fs_directory_remove (TRACKER_MINER_FS (mf),
+			                                   mount_point_file);
+			g_object_unref (mount_point_file);
+		}
+
+		g_slist_foreach (mounts_removed, (GFunc) g_free, NULL);
+		g_slist_free (mounts_removed);
+	}
+
+	/* Tell TrackerMinerFS to start monitoring the given added mount paths, if any */
+	if (mounts_added) {
+		GSList *sl;
+
+		for (sl = mounts_added; sl; sl = g_slist_next (sl)) {
+			miner_files_add_removable_or_optical_directory (mf,
+			                                                (gchar *) sl->data,
+			                                                NULL);
+		}
+
+		g_slist_foreach (mounts_added, (GFunc) g_free, NULL);
+		g_slist_free (mounts_added);
+	}
+
+	mf->private->volumes_changed_id = 0;
+
+	return FALSE;
+}
+
+static void
+index_volumes_changed_cb (GObject    *gobject,
+                          GParamSpec *arg1,
+                          gpointer    user_data)
 {
 	TrackerMinerFiles *miner_files = user_data;
 
-	if (miner_files->private->force_recheck_id == 0) {
-		/* Set idle so multiple changes in the config lead to one recheck */
-		miner_files->private->force_recheck_id =
-			g_idle_add (miner_files_force_recheck_idle, miner_files);
+	if (miner_files->private->volumes_changed_id == 0) {
+		/* Set idle so multiple changes in the config lead to one check */
+		miner_files->private->volumes_changed_id =
+			g_idle_add (index_volumes_changed_idle, miner_files);
 	}
 }
 
@@ -1607,7 +1870,7 @@ process_file_cb (GObject      *object,
 	mime_type = g_file_info_get_content_type (file_info);
 	urn = miner_files_get_file_urn (TRACKER_MINER_FILES (data->miner), file, &is_iri);
 
-	tracker_sparql_builder_insert_open (sparql, uri);
+	tracker_sparql_builder_insert_silent_open (sparql, TRACKER_MINER_FS_GRAPH_URN);
 
 	if (is_iri) {
 		tracker_sparql_builder_subject_iri (sparql, urn);
@@ -1738,7 +2001,7 @@ miner_files_ignore_next_update_file (TrackerMinerFS       *fs,
 	 * should NEVER be marked as tracker:writeback in the ontology! (else you break
 	 * the tracker-writeback feature) */
 
-	tracker_sparql_builder_insert_open (sparql, uri);
+	tracker_sparql_builder_insert_silent_open (sparql, TRACKER_MINER_FS_GRAPH_URN);
 
 	tracker_sparql_builder_subject_variable (sparql, "urn");
 	tracker_sparql_builder_predicate (sparql, "a");
@@ -1779,13 +2042,6 @@ should_check_mtime (TrackerConfig *config)
 {
 	gint crawling_interval;
 
-	if (G_UNLIKELY (!crawl_timestamp_file)) {
-		crawl_timestamp_file = g_build_filename (g_get_user_cache_dir (),
-		                                         "tracker",
-		                                         "crawling-timestamp.txt",
-		                                         NULL);
-	}
-
 	crawling_interval = tracker_config_get_crawling_interval (config);
 
 	g_message ("Checking whether to perform mtime checks during crawling:");
@@ -1798,18 +2054,15 @@ should_check_mtime (TrackerConfig *config)
 		return TRUE;
 	} else {
 		guint64 then, now;
-		gchar *content;
 
-		if (!g_file_get_contents (crawl_timestamp_file, &content, NULL, NULL)) {
+		then = tracker_db_manager_get_last_crawl_done ();
+		if (then < 1) {
 			g_message ("  No previous timestamp, crawling forced");
 			return TRUE;
 		}
 
 		now = (guint64) time (NULL);
 
-		then = g_ascii_strtoull (content, NULL, 10);
-		g_free (content);
-
 		if (now < then + (crawling_interval * SECONDS_PER_DAY)) {
 			g_message ("  Postponed");
 			return FALSE;
@@ -1820,26 +2073,10 @@ should_check_mtime (TrackerConfig *config)
 	}
 }
 
-static void
-save_crawling_time (void)
-{
-	GError *error = NULL;
-	gchar *content;
-
-	content = g_strdup_printf ("%" G_GUINT64_FORMAT, (guint64) time (NULL));
-
-	g_file_set_contents (crawl_timestamp_file, content, -1, &error);
-
-	if (error) {
-		g_critical ("Could not save crawling timestamp: %s", error->message);
-		g_error_free (error);
-	}
-}
-
 static void
 miner_files_finished (TrackerMinerFS *fs)
 {
-        save_crawling_time ();
+        tracker_db_manager_set_last_crawl_done (TRUE);
 }
 
 TrackerMiner *
@@ -2059,3 +2296,101 @@ tracker_miner_files_monitor_directory (GFile    *file,
          */
 	return TRUE;
 }
+
+static void
+remove_by_type_cb (GObject      *object,
+                   GAsyncResult *result,
+                   gpointer      user_data)
+{
+	GError *error = NULL;
+
+	tracker_miner_execute_update_finish (TRACKER_MINER (object), result, &error);
+
+	if (error) {
+		g_critical ("Could not remove by type: %s", error->message);
+		g_error_free (error);
+	}
+}
+
+static gboolean
+miner_files_remove_by_type (TrackerMinerFiles  *miner,
+                            TrackerStorageType  type)
+{
+	gboolean removable;
+	gboolean optical;
+
+	removable = TRACKER_STORAGE_TYPE_IS_REMOVABLE (type);
+	optical = TRACKER_STORAGE_TYPE_IS_OPTICAL (type);
+
+	/* Only remove if any of the flags was TRUE */
+	if (removable || optical) {
+		GString *queries;
+
+		g_debug ("  Removing all resources in store from %s ",
+		         optical ? "optical discs" : "removable devices");
+
+		queries = g_string_new ("");
+
+		/* Delete all resources where nie:dataSource is a volume
+		 * of the given type */
+		g_string_append_printf (queries,
+		                        "DELETE { "
+		                        "  ?f a rdfs:Resource "
+		                        "} WHERE { "
+		                        "  ?v a tracker:Volume ; "
+		                        "     tracker:isRemovable %s ; "
+		                        "     tracker:isOptical %s . "
+		                        "  ?f nie:dataSource ?v "
+		                        "}",
+		                        removable ? "true" : "false",
+		                        optical ? "true" : "false");
+
+		tracker_miner_execute_batch_update (TRACKER_MINER (miner),
+		                                    queries->str,
+		                                    NULL,
+		                                    remove_by_type_cb,
+		                                    NULL);
+
+		g_string_free (queries, TRUE);
+
+		return TRUE;
+	}
+
+	return FALSE;
+}
+
+static void
+miner_files_add_removable_or_optical_directory (TrackerMinerFiles *mf,
+                                                const gchar       *mount_path,
+                                                const gchar       *uuid)
+{
+	GFile *mount_point_file;
+
+	mount_point_file = g_file_new_for_path (mount_path);
+
+	/* UUID may be NULL, and if so, get it */
+	if (!uuid) {
+		uuid = tracker_storage_get_uuid_for_file (mf->private->storage,
+		                                          mount_point_file);
+		if (!uuid) {
+			g_critical ("Couldn't get UUID for mount point '%s'",
+			            mount_path);
+			g_object_unref (mount_point_file);
+			return;
+		}
+	}
+
+	g_object_set_qdata_full (G_OBJECT (mount_point_file),
+	                         mf->private->quark_mount_point_uuid,
+	                         g_strdup (uuid),
+	                         (GDestroyNotify) g_free);
+	g_object_set_qdata (G_OBJECT (mount_point_file),
+	                    mf->private->quark_directory_config_root,
+	                    GINT_TO_POINTER (TRUE));
+
+	g_message ("  Adding removable/optical: '%s'", mount_path);
+	tracker_miner_fs_directory_add (TRACKER_MINER_FS (mf),
+	                                mount_point_file,
+	                                TRUE);
+	g_object_unref (mount_point_file);
+}
