@@ -1,22 +1,20 @@
 /*
- * Copyright (C) 2009, Nokia (urho.konttori@nokia.com)
+ * Copyright (C) 2009, Nokia <ivan.frade@nokia.com>
  *
- * This library is free software; you can redistribute it and/or
- * modify it under the terms of the GNU Lesser General Public
- * License as published by the Free Software Foundation; either
- * version 2.1 of the License, or (at your option) any later version.
+ * This program is free software; you can redistribute it and/or
+ * modify it under the terms of the GNU General Public License
+ * as published by the Free Software Foundation; either version 2
+ * of the License, or (at your option) any later version.
  *
- * This library is distributed in the hope that it will be useful,
+ * This program is distributed in the hope that it will be useful,
  * but WITHOUT ANY WARRANTY; without even the implied warranty of
- * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
- * Lesser General Public License for more details.
+ * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
+ * GNU General Public License for more details.
  *
- * You should have received a copy of the GNU Lesser General Public
- * License along with this library; if not, write to the
- * Free Software Foundation, Inc., 51 Franklin Street, Fifth Floor,
- * Boston, MA  02110-1301, USA.
- *
- * Authors: Martyn Russell <martyn@imendio.com>
+ * You should have received a copy of the GNU General Public License
+ * along with this program; if not, write to the Free Software
+ * Foundation, Inc., 51 Franklin Street, Fifth Floor, Boston, MA
+ * 02110-1301, USA.
  */
 
 #include "config.h"
@@ -99,7 +97,7 @@
 	"OFFSET 0 LIMIT %d"
 #define APP_QUERY	  \
 	"SELECT" \
-	"  ?urn ?title ?tooltip ?link fts:rank(?urn) " \
+	"  ?urn ?title ?tooltip ?link fts:rank(?urn) nfo:softwareIcon(?urn)" \
 	"WHERE {" \
 	"  ?urn a nfo:Software ;" \
 	"  nie:title ?title ;" \
@@ -210,6 +208,7 @@ typedef struct {
 	gchar *title;
 	gchar *tooltip;
 	gchar *link;
+	gchar *icon_name;
 	TrackerCategory category;
 } ItemData;
 
@@ -257,6 +256,7 @@ enum {
 	COL_TITLE,
 	COL_TOOLTIP,
 	COL_LINK,
+	COL_ICON_NAME,
 	COL_COUNT
 };
 
@@ -506,6 +506,16 @@ results_window_key_press_event (GtkWidget   *widget,
 
 	priv = TRACKER_RESULTS_WINDOW_GET_PRIVATE (widget);
 
+        if (event->keyval != GDK_Return &&
+            (*event->string != '\0' ||
+             event->keyval == GDK_BackSpace)) {
+                GtkWidget *entry;
+
+                entry = tracker_aligned_window_get_widget (TRACKER_ALIGNED_WINDOW (widget));
+                gtk_propagate_event (entry, (GdkEvent *) event);
+                return TRUE;
+        }
+
 	if (GTK_WIDGET_CLASS (tracker_results_window_parent_class)->key_press_event (widget, event)) {
 		return TRUE;
 	}
@@ -589,6 +599,7 @@ item_data_new (const gchar     *urn,
                const gchar     *title,
                const gchar     *tooltip,
                const gchar     *link,
+               const gchar     *icon_name,
                TrackerCategory  category)
 {
 	ItemData *id;
@@ -599,6 +610,7 @@ item_data_new (const gchar     *urn,
 	id->title = g_strdup (title);
 	id->tooltip = g_strdup (tooltip);
 	id->link = g_strdup (link);
+	id->icon_name = g_strdup (icon_name);
 	id->category = category;
 
 	return id;
@@ -611,6 +623,7 @@ item_data_free (ItemData *id)
 	g_free (id->title);
 	g_free (id->tooltip);
 	g_free (id->link);
+	g_free (id->icon_name);
 
 	g_slice_free (ItemData, id);
 }
@@ -726,6 +739,7 @@ category_from_string (const gchar *type,
 static GdkPixbuf *
 pixbuf_get (TrackerResultsWindow *window,
             const gchar          *uri,
+            const gchar          *icon_name,
             TrackerCategory       category)
 {
 	TrackerResultsWindowPrivate *priv;
@@ -739,7 +753,39 @@ pixbuf_get (TrackerResultsWindow *window,
 	priv = TRACKER_RESULTS_WINDOW_GET_PRIVATE (window);
 	file = g_file_new_for_uri (uri);
 
-	if (category & CATEGORY_IMAGE) {
+        if (category & CATEGORY_TAG) {
+                icon_name = GTK_STOCK_INDEX;
+        } else if (category & CATEGORY_BOOKMARK) {
+                icon_name = "user-bookmarks";
+        }
+
+	if (icon_name) {
+		if (strrchr (icon_name, '.') == NULL) {
+			pixbuf = gtk_icon_theme_load_icon (priv->icon_theme,
+			                                   icon_name, 24,
+			                                   GTK_ICON_LOOKUP_USE_BUILTIN,
+			                                   &error);
+
+			if (error) {
+				g_printerr ("Couldn't get icon name '%s': %s\n",
+				            icon_name, error->message);
+				g_error_free (error);
+			}
+		} else {
+			const gchar * const *xdg_dirs;
+			gchar *path;
+			gint i;
+
+			/* Icon name is actually some filename in $(sharedir)/icons */
+			xdg_dirs = g_get_system_data_dirs ();
+
+			for (i = 0; !pixbuf && xdg_dirs[i]; i++) {
+				path = g_build_filename (xdg_dirs[i], "icons", icon_name, NULL);
+				pixbuf = gdk_pixbuf_new_from_file_at_size (path, 24, 24, NULL);
+				g_free (path);
+			}
+		}
+        } else if (category & CATEGORY_IMAGE) {
 		gchar *path;
 
 		path = g_file_get_path (file);
@@ -864,17 +910,19 @@ model_pixbuf_cell_data_func (GtkTreeViewColumn    *tree_column,
 
 	if (!requested) {
 		TrackerCategory category = CATEGORY_NONE;
-		gchar *urn;
+		gchar *urn, *icon_name;
 
 		gtk_tree_model_get (model, iter,
 		                    COL_CATEGORY_ID, &category,
 		                    COL_URN, &urn,
+				    COL_ICON_NAME, &icon_name,
 		                    -1);
 
 		if (urn) {
 			/* FIXME: Should use category */
-			pixbuf = pixbuf_get (window, urn, category);
+			pixbuf = pixbuf_get (window, urn, icon_name, category);
 			g_free (urn);
+			g_free (icon_name);
 		}
 
 		/* Cache it in the store */
@@ -968,7 +1016,8 @@ model_set_up (TrackerResultsWindow *window)
 	                            G_TYPE_STRING,         /* URN */
 	                            G_TYPE_STRING,         /* Title */
 	                            G_TYPE_STRING,         /* Tooltip */
-	                            G_TYPE_STRING);        /* Link */
+	                            G_TYPE_STRING,         /* Link */
+				    G_TYPE_STRING);	   /* Icon name */
 
 	gtk_tree_view_set_model (view, GTK_TREE_MODEL (store));
 
@@ -1011,6 +1060,10 @@ model_set_up (TrackerResultsWindow *window)
 	                                         model_pixbuf_cell_data_func,
 	                                         window,
 	                                         NULL);
+        g_object_set (cell,
+                      "height", 24,
+                      "width", 24,
+                      NULL);
 
 	cell = gtk_cell_renderer_text_new ();
 	g_object_set (cell,
@@ -1044,7 +1097,8 @@ model_add (TrackerResultsWindow *window,
            const gchar          *urn,
            const gchar          *title,
            const gchar          *tooltip,
-           const gchar          *link)
+           const gchar          *link,
+           const gchar          *icon_name)
 {
 	TrackerResultsWindowPrivate *priv;
 	GtkTreeIter iter;
@@ -1061,6 +1115,7 @@ model_add (TrackerResultsWindow *window,
 	                    COL_TITLE, title,
 	                    COL_TOOLTIP, tooltip,
 	                    COL_LINK, link,
+			    COL_ICON_NAME, icon_name,
 	                    -1);
 
 	/* path = gtk_tree_model_get_path (GTK_TREE_MODEL (window->store), &iter); */
@@ -1104,7 +1159,7 @@ search_get_foreach (gpointer value,
 	SearchQuery *sq;
 	ItemData *id;
 	gchar **metadata;
-	const gchar *urn, *title, *tooltip, *link, *rank;
+	const gchar *urn, *title, *tooltip, *link, *rank, *icon_name;
 
 	sq = user_data;
 	metadata = value;
@@ -1114,18 +1169,23 @@ search_get_foreach (gpointer value,
 	tooltip = metadata[2];
 	link = metadata[3];
 	rank = metadata[4];
+	icon_name = metadata[5];
 
 	/* App queries don't return rank or belongs */
 	if (!rank) {
 		rank = "0.0";
 	}
 
+	if (icon_name && g_str_has_prefix (icon_name, "urn:theme-icon:")) {
+		icon_name += strlen ("urn:theme-icon:");
+	}
+
 	g_print ("urn:'%s' found (rank:'%s')\n", urn, rank);
 	g_print ("  title:'%s'\n", title);
 	g_print ("  tooltip:'%s'\n", tooltip);
 	g_print ("  link:'%s'\n", link);
 
-	id = item_data_new (urn, title, tooltip, link, sq->category);
+	id = item_data_new (urn, title, tooltip, link, icon_name, sq->category);
 	sq->results = g_slist_append (sq->results, id);
 
 	/* category_from_string (type, &id->categories); */
@@ -1186,7 +1246,7 @@ search_get_cb (GPtrArray *results,
 
 			/* Add separator */
 			if (priv->first_category_populated) {
-				model_add (window, CATEGORY_NONE, NULL, NULL, NULL, NULL);
+				model_add (window, CATEGORY_NONE, NULL, NULL, NULL, NULL, NULL);
 			}
 
 			for (l = sq->results; l; l = l->next) {
@@ -1197,7 +1257,8 @@ search_get_cb (GPtrArray *results,
 				           id->urn,
 				           id->title,
 				           id->tooltip,
-				           id->link);
+				           id->link,
+					   id->icon_name);
 			}
 
 			priv->first_category_populated = TRUE;
@@ -1298,8 +1359,6 @@ search_start (TrackerResultsWindow *window)
 		return;
 	}
 
-	gtk_widget_show (GTK_WIDGET (window));
-
 	priv->first_category_populated = FALSE;
 
 	/* SPARQL requests */
@@ -1363,12 +1422,22 @@ tracker_results_window_new (GtkWidget   *parent,
 void
 tracker_results_window_popup (TrackerResultsWindow *window)
 {
-	g_return_if_fail (TRACKER_IS_RESULTS_WINDOW (window));
+        TrackerResultsWindowPrivate *priv;
+        GtkAdjustment *vadj, *hadj;
+
+        g_return_if_fail (TRACKER_IS_RESULTS_WINDOW (window));
+
+	priv = TRACKER_RESULTS_WINDOW_GET_PRIVATE (window);
 
 	gtk_widget_realize (GTK_WIDGET (window));
 	gtk_widget_show (GTK_WIDGET (window));
 
-	if (0) {
-		g_idle_add ((GSourceFunc) grab_popup_window, window);
-	}
+        /* Force scroll to top-left */
+        vadj = gtk_scrolled_window_get_vadjustment (GTK_SCROLLED_WINDOW (priv->scrolled_window));
+        gtk_adjustment_set_value (vadj, vadj->lower);
+
+        hadj = gtk_scrolled_window_get_hadjustment (GTK_SCROLLED_WINDOW (priv->scrolled_window));
+        gtk_adjustment_set_value (hadj, hadj->lower);
+
+        g_idle_add ((GSourceFunc) grab_popup_window, window);
 }
